from django.db import models
from django import forms
from django.utils.translation import ugettext as _
from django.db.models.signals import post_save
#from django.forms import widgets
from imaging.models import Image
from imaging.widgets import ImageSelection
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

class CommaSeparatedIntField(forms.CharField):
  widget = ImageSelection
  def clean(self, value):
    value = super(CommaSeparatedIntField, self).clean(value)
    if value == u'':
      return value
    valuelist = ['0','1','2','3','4','5','6','7','8','9', ',']
    for char in value:
      if char not in valuelist:
        raise forms.ValidationError(_("Image field error, please contact administrator"))
    return value
        
def save_relations(sender, **kwargs):
  '''Take care about image order and image connection to a model'''
  value_dict = getattr(kwargs['instance'], kwargs['instance']._imaging_fname).split(",")
  for i, v in enumerate(value_dict):
    try:
      image = Image.objects.get(pk=v)
      image.ordering = i
      image.content_object = kwargs['instance']
      image.save()
    except:
      pass


class ImagingField(models.Field):
  __metaclass__ = models.SubfieldBase

  def __init__(self, *args, **kwargs):
    kwargs['default'] = ''
    super(ImagingField, self).__init__(*args, **kwargs)

  def contribute_to_class(self, cls, name):
    super(ImagingField, self).contribute_to_class(cls, name)
    #content_type = ContentType.objects.get_for_model(cls)
    setattr(cls, self.name+"_set2", generic.GenericRelation(Image))

  def pre_save(self, model_instance, add):
    # set a new attribute which hold the field from which we take id later on
    setattr(model_instance, '_imaging_fname', self.name)
    # connect a new signal
    post_save.connect(save_relations, sender=model_instance.__class__)
    return getattr(model_instance, self.attname)

  def db_type(self):
    return 'char(255)'
  
  def formfield(self, **kwargs):
    defaults = {'form_class': CommaSeparatedIntField}
    defaults.update(kwargs)
    return super(ImagingField, self).formfield(**defaults)

try:
  from south.modelsinspector import add_introspection_rules
  add_introspection_rules([
    (
      [ImagingField], # Class(es) these apply to
      [],         # Positional arguments (not used)
      {},           # Keyword argument
    ),
  ], ["^imaging\.fields\.ImagingField"])
except ImportError:
  pass


from django.utils.html import escape, conditional_escape
from django.utils.translation import ugettext
from django.utils.encoding import StrAndUnicode, force_unicode
from django.utils.safestring import mark_safe
from django.forms.util import flatatt
from django.forms.widgets import Widget

from imaging.models import Image

class ImageSelection(Widget):
  class Media:
    css = { 'all' : ('imaging/imageselection.css',), }
    js = ('imaging/jquery-1.3.2.min.js',
        'imaging/jquery-ui-1.7.1.custom.min.js',
        'imaging/jquery.json-1.3.min.js',
        'imaging/imageselection.js')

  def render(self, name, value, attrs=None):
    if not value == None:
      value = value.replace(' ', '')
    if value == None or value == '': 
      initial_values = []
    else:
      ids = value.split(",")
      initial_values = Image.objects.filter(pk__in=ids)
    value = force_unicode(value)
    final_attrs = self.build_attrs(attrs, name=name)
    images_html = ''
    if len(initial_values) > 0:
      for image in initial_values:
        images_html += '<div class="image_block"><span class="image_id">'+str(image.pk)+'</span><a href="/admin/imaging/image/'+str(image.pk)+'/" target="_blank" class="edit_image">Edit</a><a href="#" class="delete_image">Delete</a><img src="'+image.imaging_thumbnail.url+'" alt="'+image.alt+'" /><br /><span class="image_name">'+image.name[:25]+'</span></div>'

    html = '''
    <div class="image_container"> %s </div>

    <div class="iframe_container">
      <iframe src="/imaging/iframe_form/" class="image_upload_form" width="100%%" height="205" frameborder="0"></iframe>
    </div>
    ''' % images_html

    return mark_safe(u'<input type="hidden" class="imaging_data" %s value="%s" /> %s' % (flatatt(final_attrs),
                conditional_escape(force_unicode(value)), html))



from django.test import TestCase
from django.contrib.auth.models import User
from django.conf import settings
from imaging.models import DEFAULT_IMAGING_SETTINGS, Image
from django.core.urlresolvers import reverse
import os
import stat
import sys

try:
  IMAGING_SETTINGS = settings.IMAGING_SETTINGS
except:
  IMAGING_SETTINGS = DEFAULT_IMAGING_SETTINGS

PERMS = "777"

class TestImagingSetup(TestCase):
  def test_file_permissions(self):
    path = settings.MEDIA_ROOT+"/"+IMAGING_SETTINGS['image_dir']
    filemode = stat.S_IMODE(os.stat(path).st_mode)
    self.assertEquals(filemode, int(PERMS, 8))

class TestAjax(TestCase):
  urls = 'imaging.urls'
  fixtures = ['users.json']

  def setUp(self):
    self.is_logged = self.client.login(username="admin", password="nimda")
    self.upload_path = settings.MEDIA_ROOT+"/"+IMAGING_SETTINGS['image_dir']

  def test_image_upload(self):
    testfile = open(settings.MEDIA_ROOT+"/imaging/imaging_testfile.jpg")
    data = {
        'name': 'Test Image: guilt',
        'alt': 'Alt of guilt',
        'title': 'Title of guilt',
        'image': testfile,
        }
    response = self.client.post(reverse('imaging_iframe_form'), data, follow=True)
    testfile.close()

    # clean up test uploads
    uploaded = os.remove(self.upload_path+"/imaging_testfile.jpg")
    uploaded = os.remove(self.upload_path+"/imaging_thumbnail/imaging_testfile.jpg")
    uploaded = os.remove(self.upload_path+"/small_thumbnail/imaging_testfile.jpg")

    self.assertEquals(self.is_logged, True)
    self.assertEquals(response.status_code, 200)
    self.assertNotEqual(response.context['callback'], None)

#TODO: figoure out why there is no new image
# def test_image_removal(self):
#   data = {
#       'id': 1
#       }
#   response = self.client.post(reverse('imaging_image_removal'), data, follow=True)
#   self.assertContains(response, 'ok', status_code=200)



import os
from setuptools import setup, find_packages

def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except:
        pass

setup(
        name = "django-imaging",
        version = "0.9.9",
        author = "Jakub Nawalaniec",
        author_email = "pielgrzym@prymityw.pl",
        description = ("AJAX driven gallery field for django"),
        license = "GNU GPL v2",
        keywords = "django",
        url = "https://github.com/pielgrzym/django-imaging",
        packages = find_packages(),
        zip_safe = False,
        include_package_data=True,
        long_description = read("OLD_README.txt"),
        classifiers=[
            "Development Status :: 3 - Alpha",
            "Framework :: Django",
            "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
            ],
        )



from django import forms
from imaging.models import Image

class AjaxUploadForm(forms.ModelForm):
  class Meta:
    model = Image
    exclude = ( 'content_type', 'content_object', 'object_id', 'ordering')
